@@ -1,17 +1,21 @@
 """Get details for a ticket."""
 # :license: MIT, see LICENSE for more details.
 
+import click
+
 import SoftLayer
 from SoftLayer.CLI import environment
 from SoftLayer.CLI import helpers
 from SoftLayer.CLI import ticket
 
-import click
-
 
 @click.command()
 @click.argument('identifier')
-@click.option('--count', type=click.INT, help="Number of updates", default=10)
+@click.option('--count',
+              type=click.INT,
+              help="Number of updates",
+              show_default=True,
+              default=10)
 @environment.pass_env
 def cli(env, identifier, count):
     """Get details for a ticket."""
