@@ -5,11 +5,17 @@
 
     :license: MIT, see LICENSE for more details.
 """
+import logging
 import socket
+import time
 
 import SoftLayer
+from SoftLayer.decoration import retry
 from SoftLayer.managers import ordering
 from SoftLayer import utils
+
+LOGGER = logging.getLogger(__name__)
+
 # Invalid names are ignored due to long method names and short argument names
 # pylint: disable=invalid-name, no-self-use
 
@@ -82,6 +88,7 @@ class HardwareManager(utils.IdentifierMixin, object):
                                 immediate, False, cancel_reason, comment,
                                 id=billing_id)
 
+    @retry(logger=LOGGER)
     def list_hardware(self, tags=None, cpus=None, memory=None, hostname=None,
                       domain=None, datacenter=None, nic_speed=None,
                       public_ip=None, private_ip=None, **kwargs):
@@ -169,6 +176,7 @@ class HardwareManager(utils.IdentifierMixin, object):
         kwargs['filter'] = _filter.to_dict()
         return self.account.getHardware(**kwargs)
 
+    @retry(logger=LOGGER)
     def get_hardware(self, hardware_id, **kwargs):
         """Get details about a hardware device.
 
@@ -290,7 +298,7 @@ class HardwareManager(utils.IdentifierMixin, object):
 
         See get_create_options() for valid arguments.
 
-        :param string size: server size name
+        :param string size: server size name or presetId
         :param string hostname: server hostname
         :param string domain: server domain name
         :param string location: location (datacenter) name
@@ -335,6 +343,7 @@ class HardwareManager(utils.IdentifierMixin, object):
             'moving': 'Moving to competitor',
         }
 
+    @retry(logger=LOGGER)
     def get_create_options(self):
         """Returns valid options for ordering hardware."""
 
@@ -395,29 +404,26 @@ class HardwareManager(utils.IdentifierMixin, object):
             'extras': extras,
         }
 
+    @retry(logger=LOGGER)
     def _get_package(self):
         """Get the package related to simple hardware ordering."""
         mask = '''
-items[
-    keyName,
-    capacity,
-    description,
-    attributes[id,attributeTypeKeyName],
-    itemCategory[id,categoryCode],
-    softwareDescription[id,referenceCode,longDescription],
-    prices
-],
-activePresets,
-regions[location[location[priceGroups]]]
-'''
+            items[
+                keyName,
+                capacity,
+                description,
+                attributes[id,attributeTypeKeyName],
+                itemCategory[id,categoryCode],
+                softwareDescription[id,referenceCode,longDescription],
+                prices
+            ],
+            activePresets,
+            regions[location[location[priceGroups]]]
+            '''
 
-        package_type = 'BARE_METAL_CPU_FAST_PROVISION'
-        packages = self.ordering_manager.get_packages_of_type([package_type],
-                                                              mask=mask)
-        if len(packages) != 1:
-            raise SoftLayer.SoftLayerError("Ordering package not found")
-
-        return packages[0]
+        package_keyname = 'BARE_METAL_SERVER'
+        package = self.ordering_manager.get_package_by_key(package_keyname, mask=mask)
+        return package
 
     def _generate_create_dict(self,
                               size=None,
@@ -490,7 +496,7 @@ regions[location[location[priceGroups]]]
         results = self.list_hardware(hostname=hostname, mask="id")
         return [result['id'] for result in results]
 
-    def _get_ids_from_ip(self, ip):
+    def _get_ids_from_ip(self, ip):  # pylint: disable=inconsistent-return-statements
         """Returns list of matching hardware IDs for a given ip address."""
         try:
             # Does it look like an ip address?
@@ -574,8 +580,33 @@ regions[location[location[priceGroups]]]
         """
 
         return self.hardware.createFirmwareUpdateTransaction(
-            bool(ipmi), bool(raid_controller), bool(bios), bool(hard_drive),
-            id=hardware_id)
+            bool(ipmi), bool(raid_controller), bool(bios), bool(hard_drive), id=hardware_id)
+
+    def wait_for_ready(self, instance_id, limit=14400, delay=10, pending=False):
+        """Determine if a Server is ready.
+
+        A server is ready when no transactions are running on it.
+
+        :param int instance_id: The instance ID with the pending transaction
+        :param int limit: The maximum amount of seconds to wait.
+        :param int delay: The number of seconds to sleep before checks. Defaults to 10.
+        """
+        now = time.time()
+        until = now + limit
+        mask = "mask[id, lastOperatingSystemReload[id], activeTransaction, provisionDate]"
+        instance = self.get_hardware(instance_id, mask=mask)
+        while now <= until:
+            if utils.is_ready(instance, pending):
+                return True
+            transaction = utils.lookup(instance, 'activeTransaction', 'transactionStatus', 'friendlyName')
+            snooze = min(delay, until - now)
+            LOGGER.info("%s - %d not ready. Auto retry in %ds", transaction, instance_id, snooze)
+            time.sleep(snooze)
+            instance = self.get_hardware(instance_id, mask=mask)
+            now = time.time()
+
+        LOGGER.info("Waiting for %d expired.", instance_id)
+        return False
 
 
 def _get_extra_price_id(items, key_name, hourly, location):
@@ -738,15 +769,13 @@ def _get_location(package, location):
         if region['location']['location']['name'] == location:
             return region
 
-    raise SoftLayer.SoftLayerError("Could not find valid location for: '%s'"
-                                   % location)
+    raise SoftLayer.SoftLayerError("Could not find valid location for: '%s'" % location)
 
 
 def _get_preset_id(package, size):
     """Get the preset id given the keyName of the preset."""
     for preset in package['activePresets']:
-        if preset['keyName'] == size:
+        if preset['keyName'] == size or preset['id'] == size:
             return preset['id']
 
-    raise SoftLayer.SoftLayerError("Could not find valid size for: '%s'"
-                                   % size)
+    raise SoftLayer.SoftLayerError("Could not find valid size for: '%s'" % size)
