@@ -3,6 +3,7 @@
     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
     :license: MIT, see LICENSE for more details.
+
 """
 import mock
 
@@ -191,7 +192,8 @@ class VSTests(testing.TestCase):
                   'localDiskFlag': True,
                   'maxMemory': 1024,
                   'hostname': 'server',
-                  'startCpus': 1}],)
+                  'startCpus': 1,
+                  'supplementalCreateObjectOptions': {'bootMode': None}}],)
         self.assert_called_with('SoftLayer_Virtual_Guest', 'createObjects',
                                 args=args)
         self.assert_called_with('SoftLayer_Virtual_Guest', 'setTags',
@@ -231,6 +233,7 @@ class VSTests(testing.TestCase):
             'localDiskFlag': True,
             'operatingSystemReferenceCode': "STRING",
             'hourlyBillingFlag': True,
+            'supplementalCreateObjectOptions': {'bootMode': None},
         }
 
         self.assertEqual(data, assert_data)
@@ -253,6 +256,7 @@ class VSTests(testing.TestCase):
             'domain': 'example.com',
             'localDiskFlag': True,
             'operatingSystemReferenceCode': "STRING",
+            'supplementalCreateObjectOptions': {'bootMode': None},
         }
 
         self.assertEqual(data, assert_data)
@@ -274,6 +278,7 @@ class VSTests(testing.TestCase):
             'localDiskFlag': True,
             'blockDeviceTemplateGroup': {"globalIdentifier": "45"},
             'hourlyBillingFlag': True,
+            'supplementalCreateObjectOptions': {'bootMode': None},
         }
 
         self.assertEqual(data, assert_data)
@@ -297,6 +302,7 @@ class VSTests(testing.TestCase):
             'operatingSystemReferenceCode': "STRING",
             'hourlyBillingFlag': True,
             'dedicatedAccountHostOnlyFlag': True,
+            'supplementalCreateObjectOptions': {'bootMode': None},
         }
 
         self.assertEqual(data, assert_data)
@@ -320,6 +326,7 @@ class VSTests(testing.TestCase):
             'operatingSystemReferenceCode': "STRING",
             'hourlyBillingFlag': True,
             'datacenter': {"name": 'sng01'},
+            'supplementalCreateObjectOptions': {'bootMode': None},
         }
 
         self.assertEqual(data, assert_data)
@@ -343,6 +350,7 @@ class VSTests(testing.TestCase):
             'operatingSystemReferenceCode': "STRING",
             'hourlyBillingFlag': True,
             'primaryNetworkComponent': {"networkVlan": {"id": 1}},
+            'supplementalCreateObjectOptions': {'bootMode': None},
         }
 
         self.assertEqual(data, assert_data)
@@ -366,6 +374,7 @@ class VSTests(testing.TestCase):
             'operatingSystemReferenceCode': "STRING",
             'hourlyBillingFlag': True,
             'primaryBackendNetworkComponent': {"networkVlan": {"id": 1}},
+            'supplementalCreateObjectOptions': {'bootMode': None},
         }
 
         self.assertEqual(data, assert_data)
@@ -389,6 +398,7 @@ class VSTests(testing.TestCase):
             'operatingSystemReferenceCode': "STRING",
             'hourlyBillingFlag': True,
             'userData': [{'value': "ICANHAZVSI"}],
+            'supplementalCreateObjectOptions': {'bootMode': None},
         }
 
         self.assertEqual(data, assert_data)
@@ -412,6 +422,7 @@ class VSTests(testing.TestCase):
             'operatingSystemReferenceCode': "STRING",
             'hourlyBillingFlag': True,
             'networkComponents': [{'maxSpeed': 9001}],
+            'supplementalCreateObjectOptions': {'bootMode': None},
         }
 
         self.assertEqual(data, assert_data)
@@ -437,6 +448,7 @@ class VSTests(testing.TestCase):
             'privateNetworkOnlyFlag': True,
             'hourlyBillingFlag': True,
             'networkComponents': [{'maxSpeed': 9001}],
+            'supplementalCreateObjectOptions': {'bootMode': None},
         }
 
         self.assertEqual(data, assert_data)
@@ -460,6 +472,7 @@ class VSTests(testing.TestCase):
             'operatingSystemReferenceCode': "STRING",
             'hourlyBillingFlag': True,
             'postInstallScriptUri': 'https://example.com/boostrap.sh',
+            'supplementalCreateObjectOptions': {'bootMode': None},
         }
 
         self.assertEqual(data, assert_data)
@@ -483,6 +496,7 @@ class VSTests(testing.TestCase):
             'operatingSystemReferenceCode': "STRING",
             'hourlyBillingFlag': True,
             'sshKeys': [{'id': 543}],
+            'supplementalCreateObjectOptions': {'bootMode': None},
         }
 
         self.assertEqual(data, assert_data)
@@ -536,6 +550,29 @@ class VSTests(testing.TestCase):
         self.assertTrue(data.get('blockDevices'))
         self.assertEqual(data['blockDevices'], assert_data['blockDevices'])
 
+    def test_generate_boot_mode(self):
+        data = self.vs._generate_create_dict(
+            cpus=1,
+            memory=1,
+            hostname='test',
+            domain='example.com',
+            os_code="STRING",
+            boot_mode="HVM"
+        )
+
+        assert_data = {
+            'startCpus': 1,
+            'maxMemory': 1,
+            'hostname': 'test',
+            'domain': 'example.com',
+            'localDiskFlag': True,
+            'operatingSystemReferenceCode': "STRING",
+            'hourlyBillingFlag': True,
+            'supplementalCreateObjectOptions': {'bootMode': 'HVM'},
+        }
+
+        self.assertEqual(data, assert_data)
+
     def test_change_port_speed_public(self):
         result = self.vs.change_port_speed(1, True, 100)
 
@@ -636,13 +673,6 @@ class VSTests(testing.TestCase):
                                 identifier=1)
 
     def test_upgrade(self):
-        mock = self.set_mock('SoftLayer_Product_Package', 'getAllObjects')
-        mock.return_value = [
-            {'id': 46, 'name': 'Virtual Servers',
-             'description': 'Virtual Server Instances',
-             'type': {'keyName': 'VIRTUAL_SERVER_INSTANCE'}, 'isActive': 1},
-        ]
-
         # test single upgrade
         result = self.vs.upgrade(1, cpus=4, public=False)
 
@@ -678,10 +708,19 @@ class VSTests(testing.TestCase):
         self.assertIn({'id': 1122}, order_container['prices'])
         self.assertEqual(order_container['virtualGuests'], [{'id': 1}])
 
-    def test_upgrade_skips_location_based_prices(self):
-        # Test that no prices that have locationGroupId set are used
-        self.assertRaises(exceptions.SoftLayerError,
-                          self.vs.upgrade, 1, cpus=55, memory=2, public=True)
+    def test_upgrade_dedicated_host_instance(self):
+        mock = self.set_mock('SoftLayer_Virtual_Guest', 'getUpgradeItemPrices')
+        mock.return_value = fixtures.SoftLayer_Virtual_Guest.DEDICATED_GET_UPGRADE_ITEM_PRICES
+
+        # test single upgrade
+        result = self.vs.upgrade(1, cpus=4, public=False)
+
+        self.assertEqual(result, True)
+        self.assert_called_with('SoftLayer_Product_Order', 'placeOrder')
+        call = self.calls('SoftLayer_Product_Order', 'placeOrder')[0]
+        order_container = call.args[0]
+        self.assertEqual(order_container['prices'], [{'id': 115566}])
+        self.assertEqual(order_container['virtualGuests'], [{'id': 1}])
 
     def test_get_item_id_for_upgrade(self):
         item_id = 0
@@ -693,6 +732,42 @@ class VSTests(testing.TestCase):
                 break
         self.assertEqual(1133, item_id)
 
+    def test_get_package_items(self):
+        self.vs._get_package_items()
+        self.assert_called_with('SoftLayer_Product_Package', 'getItems')
+
+    def test_get_price_id_for_upgrade(self):
+        package_items = self.vs._get_package_items()
+
+        price_id = self.vs._get_price_id_for_upgrade(package_items=package_items,
+                                                     option='cpus',
+                                                     value='4')
+        self.assertEqual(1144, price_id)
+
+    def test_get_price_id_for_upgrade_skips_location_price(self):
+        package_items = self.vs._get_package_items()
+
+        price_id = self.vs._get_price_id_for_upgrade(package_items=package_items,
+                                                     option='cpus',
+                                                     value='55')
+        self.assertEqual(None, price_id)
+
+    def test_get_price_id_for_upgrade_finds_nic_price(self):
+        package_items = self.vs._get_package_items()
+
+        price_id = self.vs._get_price_id_for_upgrade(package_items=package_items,
+                                                     option='memory',
+                                                     value='2')
+        self.assertEqual(1133, price_id)
+
+    def test_get_price_id_for_upgrade_finds_memory_price(self):
+        package_items = self.vs._get_package_items()
+
+        price_id = self.vs._get_price_id_for_upgrade(package_items=package_items,
+                                                     option='nic_speed',
+                                                     value='1000')
+        self.assertEqual(1122, price_id)
+
 
 class VSWaitReadyGoTests(testing.TestCase):
 
@@ -705,13 +780,11 @@ class VSWaitReadyGoTests(testing.TestCase):
     def test_wait_interface(self, ready):
         # verify interface to wait_for_ready is intact
         self.vs.wait_for_transaction(1, 1)
-        ready.assert_called_once_with(1, 1, delay=1, pending=True)
+        ready.assert_called_once_with(1, 1, delay=10, pending=True)
 
     def test_active_not_provisioned(self):
         # active transaction and no provision date should be false
-        self.guestObject.side_effect = [
-            {'activeTransaction': {'id': 1}},
-        ]
+        self.guestObject.return_value = {'activeTransaction': {'id': 1}}
         value = self.vs.wait_for_ready(1, 0)
         self.assertFalse(value)
 
@@ -724,18 +797,22 @@ class VSWaitReadyGoTests(testing.TestCase):
         value = self.vs.wait_for_ready(1, 1)
         self.assertTrue(value)
 
-    def test_active_provision_pending(self):
+    @mock.patch('time.sleep')
+    @mock.patch('time.time')
+    def test_active_provision_pending(self, _now, _sleep):
+        _now.side_effect = [0, 0, 1, 1, 2, 2]
         # active transaction and provision date
         # and pending should be false
-        self.guestObject.side_effect = [
-            {'activeTransaction': {'id': 1}, 'provisionDate': 'aaa'},
-        ]
-        value = self.vs.wait_for_ready(1, 0, pending=True)
+        self.guestObject.return_value = {'activeTransaction': {'id': 2}, 'provisionDate': 'aaa'}
+
+        value = self.vs.wait_for_ready(instance_id=1, limit=1, delay=1, pending=True)
+        _sleep.assert_has_calls([mock.call(0)])
         self.assertFalse(value)
 
     def test_active_reload(self):
         # actively running reload
         self.guestObject.side_effect = [
+            {'activeTransaction': {'id': 1}},
             {
                 'activeTransaction': {'id': 1},
                 'provisionDate': 'aaa',
@@ -747,47 +824,40 @@ class VSWaitReadyGoTests(testing.TestCase):
 
     def test_reload_no_pending(self):
         # reload complete, maintance transactions
-        self.guestObject.side_effect = [
-            {
-                'activeTransaction': {'id': 2},
-                'provisionDate': 'aaa',
-                'lastOperatingSystemReload': {'id': 1},
-            },
-        ]
+        self.guestObject.return_value = {
+            'activeTransaction': {'id': 2},
+            'provisionDate': 'aaa',
+            'lastOperatingSystemReload': {'id': 1},
+        }
+
         value = self.vs.wait_for_ready(1, 1)
         self.assertTrue(value)
 
-    def test_reload_pending(self):
+    @mock.patch('time.sleep')
+    @mock.patch('time.time')
+    def test_reload_pending(self, _now, _sleep):
+        _now.side_effect = [0, 0, 1, 1, 2, 2]
         # reload complete, pending maintance transactions
-        self.guestObject.side_effect = [
-            {
-                'activeTransaction': {'id': 2},
-                'provisionDate': 'aaa',
-                'lastOperatingSystemReload': {'id': 1},
-            },
-        ]
-        value = self.vs.wait_for_ready(1, 0, pending=True)
+        self.guestObject.return_value = {'activeTransaction': {'id': 2},
+                                         'provisionDate': 'aaa',
+                                         'lastOperatingSystemReload': {'id': 1}}
+        value = self.vs.wait_for_ready(instance_id=1, limit=1, delay=1, pending=True)
+        _sleep.assert_has_calls([mock.call(0)])
         self.assertFalse(value)
 
     @mock.patch('time.sleep')
     def test_ready_iter_once_incomplete(self, _sleep):
-        self.guestObject = self.client['Virtual_Guest'].getObject
-
         # no iteration, false
-        self.guestObject.side_effect = [
-            {'activeTransaction': {'id': 1}},
-        ]
-        value = self.vs.wait_for_ready(1, 0)
+        self.guestObject.return_value = {'activeTransaction': {'id': 1}}
+        value = self.vs.wait_for_ready(1, 0, delay=1)
         self.assertFalse(value)
-        self.assertFalse(_sleep.called)
+        _sleep.assert_has_calls([mock.call(0)])
 
     @mock.patch('time.sleep')
     def test_iter_once_complete(self, _sleep):
         # no iteration, true
-        self.guestObject.side_effect = [
-            {'provisionDate': 'aaa'},
-        ]
-        value = self.vs.wait_for_ready(1, 1)
+        self.guestObject.return_value = {'provisionDate': 'aaa'}
+        value = self.vs.wait_for_ready(1, 1, delay=1)
         self.assertTrue(value)
         self.assertFalse(_sleep.called)
 
@@ -801,7 +871,7 @@ class VSWaitReadyGoTests(testing.TestCase):
             {'provisionDate': 'aaa'},
         ]
 
-        value = self.vs.wait_for_ready(1, 4)
+        value = self.vs.wait_for_ready(1, 4, delay=1)
         self.assertTrue(value)
         _sleep.assert_has_calls([mock.call(1), mock.call(1), mock.call(1)])
         self.guestObject.assert_has_calls([
@@ -814,14 +884,16 @@ class VSWaitReadyGoTests(testing.TestCase):
     def test_iter_two_incomplete(self, _sleep, _time):
         # test 2 iterations, with no matches
         self.guestObject.side_effect = [
+            {'activeTransaction': {'id': 1}},
             {'activeTransaction': {'id': 1}},
             {'activeTransaction': {'id': 1}},
             {'provisionDate': 'aaa'}
         ]
-        _time.side_effect = [0, 1, 2]
-        value = self.vs.wait_for_ready(1, 2)
+        # logging calls time.time as of pytest3.3, not sure if there is a better way of getting around that.
+        _time.side_effect = [0, 1, 2, 3, 4, 5, 6]
+        value = self.vs.wait_for_ready(1, 2, delay=1)
         self.assertFalse(value)
-        _sleep.assert_called_once_with(1)
+        _sleep.assert_has_calls([mock.call(1), mock.call(0)])
         self.guestObject.assert_has_calls([
             mock.call(id=1, mask=mock.ANY),
             mock.call(id=1, mask=mock.ANY),
@@ -832,9 +904,30 @@ class VSWaitReadyGoTests(testing.TestCase):
     def test_iter_20_incomplete(self, _sleep, _time):
         """Wait for up to 20 seconds (sleeping for 10 seconds) for a server."""
         self.guestObject.return_value = {'activeTransaction': {'id': 1}}
-        _time.side_effect = [0, 10, 20]
+        # logging calls time.time as of pytest3.3, not sure if there is a better way of getting around that.
+        _time.side_effect = [0, 0, 10, 10, 20, 20, 50, 60]
         value = self.vs.wait_for_ready(1, 20, delay=10)
         self.assertFalse(value)
         self.guestObject.assert_has_calls([mock.call(id=1, mask=mock.ANY)])
 
         _sleep.assert_has_calls([mock.call(10)])
+
+    @mock.patch('SoftLayer.decoration.sleep')
+    @mock.patch('SoftLayer.transports.FixtureTransport.__call__')
+    @mock.patch('time.time')
+    @mock.patch('time.sleep')
+    def test_exception_from_api(self, _sleep, _time, _vs, _dsleep):
+        """Tests escalating scale back when an excaption is thrown"""
+        _dsleep.return_value = False
+
+        self.guestObject.side_effect = [
+            exceptions.TransportError(104, "Its broken"),
+            {'activeTransaction': {'id': 1}},
+            {'provisionDate': 'aaa'}
+        ]
+        # logging calls time.time as of pytest3.3, not sure if there is a better way of getting around that.
+        _time.side_effect = [0, 1, 2, 3, 4]
+        value = self.vs.wait_for_ready(1, 20, delay=1)
+        _sleep.assert_called_once()
+        _dsleep.assert_called_once()
+        self.assertTrue(value)
